local ADDON_NAME, ADDON = ...
local MODULE_NAME = "addon.lua"

ADDON[1] = {} -- TH, Addon
ADDON[2] = {} -- C, Config
ADDON[3] = {} -- L, Locale
local TH, C, L = unpack(ADDON)

--[===[@debug@
_G["TH"] = TH --Expose internals on the Plugin lib
--@end-debug@]==]]===]


---------------------------------------------
-- CONSTANTS
---------------------------------------------
TH.ADDON_NAME = ADDON_NAME
TH.VERSION = GetAddOnMetadata(ADDON_NAME, "Version")


---------------------------------------------
-- EVENTS & CALLBACKS
---------------------------------------------
TH.callbacks = LibStub("CallbackHandler-1.0"):New(TH)
TH.eventCallbacks = LibStub("CallbackHandler-1.0"):New(TH, "RegisterEvent", "UnregisterEvent", false)

local frame = CreateFrame("frame")
frame:SetScript("OnEvent", function(self, event, ...)
    TH.eventCallbacks:Fire(event, ...)
end)

function TH.eventCallbacks:OnUsed(target, event)
    frame:RegisterEvent(event)
end
function TH.eventCallbacks:OnUnused(target, event)
    frame:UnregisterEvent(event)
end

function TH:InitOnDemand(addon, init)
    if IsAddOnLoaded(addon) then
        init()
    else
        local registrationKey = addon .. "@" .. GetTime()
        TH.RegisterEvent(registrationKey, "ADDON_LOADED", function(event, addonName)
            if ( addon == addonName ) then
                TH.UnregisterEvent(registrationKey, "ADDON_LOADED")
                init()
            end
        end)
    end
end


---------------------------------------------
-- INITIALIZE
---------------------------------------------
--[===[@alpha@
LoadAddOn("Blizzard_DebugTools")
--@end-alpha@]===]

TH.RegisterEvent(MODULE_NAME, "ADDON_LOADED", function(event, addonName)
    if ( addonName == TH.ADDON_NAME ) then
        TH.UnregisterEvent(MODULE_NAME, "ADDON_LOADED")

        -- Initialize DB
        TargetHealthDB = TargetHealthDB or { version=1, forcePercentages=false }

        -- Initialize addon modules
        TH.callbacks:Fire("initialize")
    end
end)
